"""
additional meta information for all BOs or COMponents
"""
from datetime import datetime
from typing import Optional

from pydantic import BaseModel


class MetaDataMixin(BaseModel):
    """
    a mixin that adds metadata to business objects and/or components
    """

    creation_date: Optional[datetime] = None



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field

from ..com.energiemix import Energiemix
from ..com.externe_referenz import ExterneReferenz
from ..com.regionale_preisgarantie import RegionalePreisgarantie
from ..com.regionale_tarifpreisposition import RegionaleTarifpreisposition
from ..com.regionaler_auf_abschlag import RegionalerAufAbschlag
from ..com.tarifberechnungsparameter import Tarifberechnungsparameter
from ..com.tarifeinschraenkung import Tarifeinschraenkung
from ..com.vertragskonditionen import Vertragskonditionen
from ..com.zeitraum import Zeitraum
from ..enum.bo_typ import BoTyp
from ..enum.kundentyp import Kundentyp
from ..enum.sparte import Sparte
from ..enum.tarifart import Tarifart
from ..enum.tarifmerkmal import Tarifmerkmal
from ..enum.tariftyp import Tariftyp
from .marktteilnehmer import Marktteilnehmer


class Regionaltarif(BaseModel):
    """
    .. raw:: html

        <object data="../_static/images/bo4e/bo/Regionaltarif.svg" type="image/svg+xml"></object>

    .. HINT::
        `Regionaltarif JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Regionaltarif.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    anbieter: Marktteilnehmer | None = None
    anbietername: str | None = Field(default=None, title="Anbietername")
    anwendung_von: datetime | None = Field(default=None, alias="anwendungVon", title="Anwendungvon")
    bemerkung: str | None = Field(default=None, title="Bemerkung")
    berechnungsparameter: Tarifberechnungsparameter | None = None
    bezeichnung: str | None = Field(default=None, title="Bezeichnung")
    bo_typ: BoTyp | None = Field(default=BoTyp.REGIONALTARIF, alias="boTyp")
    energiemix: Energiemix | None = None
    externe_referenzen: list[ExterneReferenz] | None = Field(
        default=None, alias="externeReferenzen", title="Externereferenzen"
    )
    kundentypen: list[Kundentyp] | None = Field(default=None, title="Kundentypen")
    preisgarantien: list[RegionalePreisgarantie] | None = Field(default=None, title="Preisgarantien")
    preisstand: datetime | None = Field(default=None, title="Preisstand")
    sparte: Sparte | None = None
    tarif_auf_abschlaege: list[RegionalerAufAbschlag] | None = Field(
        default=None, alias="tarifAufAbschlaege", title="Tarifaufabschlaege"
    )
    tarifart: Tarifart | None = None
    tarifeinschraenkung: Tarifeinschraenkung | None = None
    tarifmerkmale: list[Tarifmerkmal] | None = Field(default=None, title="Tarifmerkmale")
    tarifpreise: list[RegionaleTarifpreisposition] | None = Field(default=None, title="Tarifpreise")
    tariftyp: Tariftyp | None = None
    versionstruktur: str | None = Field(default="2", title="Versionstruktur")
    vertragskonditionen: Vertragskonditionen | None = None
    website: str | None = Field(default=None, title="Website")
    zeitliche_gueltigkeit: Zeitraum | None = Field(default=None, alias="zeitlicheGueltigkeit")



from enum import Enum


class Kalkulationsmethode(str, Enum):
    """
    Auflistung der verschiedenen Berechnungsmethoden für ein Preisblatt.
    """

    STUFEN = "STUFEN"
    ZONEN = "ZONEN"
    VORZONEN_GP = "VORZONEN_GP"
    SIGMOID = "SIGMOID"
    BLINDARBEIT_GT_50_PROZENT = "BLINDARBEIT_GT_50_PROZENT"
    BLINDARBEIT_GT_40_PROZENT = "BLINDARBEIT_GT_40_PROZENT"
    BLINDARBEIT_MIT_FREIMENGE = "BLINDARBEIT_MIT_FREIMENGE"
    AP_GP_ZONEN = "AP_GP_ZONEN"
    LP_INSTALL_LEISTUNG = "LP_INSTALL_LEISTUNG"
    AP_TRANSPORT_ODER_VERTEILNETZ = "AP_TRANSPORT_ODER_VERTEILNETZ"
    AP_TRANSPORT_ODER_VERTEILNETZ_ORTSVERTEILNETZ_SIGMOID = "AP_TRANSPORT_ODER_VERTEILNETZ_ORTSVERTEILNETZ_SIGMOID"
    LP_JAHRESVERBRAUCH = "LP_JAHRESVERBRAUCH"
    LP_TRANSPORT_ODER_VERTEILNETZ = "LP_TRANSPORT_ODER_VERTEILNETZ"
    LP_TRANSPORT_ODER_VERTEILNETZ_ORTSVERTEILNETZ_SIGMOID = "LP_TRANSPORT_ODER_VERTEILNETZ_ORTSVERTEILNETZ_SIGMOID"
    FUNKTIONEN = "FUNKTIONEN"
    VERBRAUCH_UEBER_SLP_GRENZE_FUNKTIONSBEZOGEN_WEITERE_BERECHNUNG_ALS_LGK = (
        "VERBRAUCH_UEBER_SLP_GRENZE_FUNKTIONSBEZOGEN_WEITERE_BERECHNUNG_ALS_LGK"
    )



from enum import Enum


class Oekozertifikat(str, Enum):
    """
    Zertifikate für Ökostrom von verschiedenen Herausgebern.
    """

    CMS_EE01 = "CMS_EE01"
    CMS_EE02 = "CMS_EE02"
    EECS = "EECS"
    FRAUNHOFER = "FRAUNHOFER"
    BET = "BET"
    KLIMA_INVEST = "KLIMA_INVEST"
    LGA = "LGA"
    FREIBERG = "FREIBERG"
    RECS = "RECS"
    REGS_EGL = "REGS_EGL"
    TUEV = "TUEV"
    TUEV_HESSEN = "TUEV_HESSEN"
    TUEV_NORD = "TUEV_NORD"
    TUEV_RHEINLAND = "TUEV_RHEINLAND"
    TUEV_SUED = "TUEV_SUED"
    TUEV_SUED_EE01 = "TUEV_SUED_EE01"
    TUEV_SUED_EE02 = "TUEV_SUED_EE02"



from enum import Enum


class Waehrungseinheit(str, Enum):
    """
    In diesem Enum werden die Währungen und ihre Untereinheiten definiert, beispielsweise für die Verwendung in Preisen.
    """

    EUR = "EUR"
    CT = "CT"



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field

from ..com.externe_referenz import ExterneReferenz
from ..enum.bo_typ import BoTyp


class Dokument(BaseModel):
    """
    A generic document reference like for bills, order confirmations and cancellations
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    versionstruktur: str | None = Field(default="2", title="Versionstruktur")
    bo_typ: BoTyp | None = Field(default=BoTyp.GESCHAEFTSOBJEKT, alias="boTyp")
    externe_referenzen: list[ExterneReferenz] | None = Field(
        default=None, alias="externeReferenzen", title="Externereferenzen"
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    erstellungsdatum: datetime = Field(..., title="Erstellungsdatum")
    has_been_sent: bool = Field(..., alias="hasBeenSent", title="Hasbeensent")
    dokumentenname: str = Field(..., title="Dokumentenname")
    vorlagenname: str = Field(..., title="Vorlagenname")



"""
Contains the dataset for the resource loader.
It also contains the validation logic for the resource loader dataset.
"""
from ibims.bo4e import Marktlokation, Messlokation, Vertrag, Zaehler
from ibims.datasets import DataSetBaseModel


class TripicaResourceLoaderDataSet(DataSetBaseModel):
    """
    This is a bo4e data set that consists of
    * a Vertrag
    * a Marktlokation
    * a Messlokation
    * a Zaehler.
    In the context of this package is may be used to create Tripica Resources.
    """

    marktlokation: Marktlokation
    messlokation: Messlokation
    vertrag: Vertrag
    zaehler: Zaehler



from pydantic import BaseModel, ConfigDict, Field

from ..com.externe_referenz import ExterneReferenz
from ..com.standorteigenschaften_gas import StandorteigenschaftenGas
from ..com.standorteigenschaften_strom import StandorteigenschaftenStrom
from ..enum.bo_typ import BoTyp


class Standorteigenschaften(BaseModel):
    """
    Modelliert die regionalen und spartenspezifischen Eigenschaften einer gegebenen Adresse.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Standorteigenschaften.svg" type="image/svg+xml"></object>

    .. HINT::
        `Standorteigenschaften JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Standorteigenschaften.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    bo_typ: BoTyp | None = Field(default=BoTyp.STANDORTEIGENSCHAFTEN, alias="boTyp")
    eigenschaften_gas: StandorteigenschaftenGas | None = Field(default=None, alias="eigenschaftenGas")
    eigenschaften_strom: list[StandorteigenschaftenStrom] | None = Field(
        default=None, alias="eigenschaftenStrom", title="Eigenschaftenstrom"
    )
    externe_referenzen: list[ExterneReferenz] | None = Field(
        default=None, alias="externeReferenzen", title="Externereferenzen"
    )
    versionstruktur: str | None = Field(default="2", title="Versionstruktur")



from decimal import Decimal

from pydantic import BaseModel, ConfigDict, Field


class Sigmoidparameter(BaseModel):
    """
    Die Sigmoid-Funktion, beispielsweise zur Berechnung eines Leistungspreises hat die Form:
    LP=A/(1+(P/B)^C)+D

    .. raw:: html

        <object data="../_static/images/bo4e/com/Sigmoidparameter.svg" type="image/svg+xml"></object>

    .. HINT::
        `Sigmoidparameter JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Sigmoidparameter.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    a: Decimal | None = Field(default=None, alias="A", title="A")
    b: Decimal | None = Field(default=None, alias="B", title="B")
    c: Decimal | None = Field(default=None, alias="C", title="C")
    d: Decimal | None = Field(default=None, alias="D", title="D")
    id: str | None = Field(default=None, alias="_id", title=" Id")



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field

from ..com.adresse import Adresse
from ..com.externe_referenz import ExterneReferenz
from ..enum.anrede import Anrede
from ..enum.bo_typ import BoTyp
from ..enum.geschaeftspartnerrolle import Geschaeftspartnerrolle
from ..enum.kontaktart import Kontaktart


class Geschaeftspartner(BaseModel):
    """
    Mit diesem Objekt können Geschäftspartner übertragen werden.
    Sowohl Unternehmen, als auch Privatpersonen können Geschäftspartner sein.
    Hinweis: Marktteilnehmer haben ein eigenes BO, welches sich von diesem BO ableitet.
    Hier sollte daher keine Zuordnung zu Marktrollen erfolgen.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Geschaeftspartner.svg" type="image/svg+xml"></object>

    .. HINT::
        `Geschaeftspartner JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Geschaeftspartner.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    amtsgericht: str | None = Field(default=None, title="Amtsgericht")
    anrede: Anrede | None = None
    bo_typ: BoTyp | None = Field(default=BoTyp.GESCHAEFTSPARTNER, alias="boTyp")
    e_mail_adresse: str | None = Field(default=None, alias="eMailAdresse", title="Emailadresse")
    externe_referenzen: list[ExterneReferenz] | None = Field(
        default=None, alias="externeReferenzen", title="Externereferenzen"
    )
    geschaeftspartnerrolle: list[Geschaeftspartnerrolle] | None = Field(default=None, title="Geschaeftspartnerrolle")
    gewerbekennzeichnung: bool | None = Field(default=None, title="Gewerbekennzeichnung")
    glaeubiger_id: str | None = Field(default=None, alias="glaeubigerId", title="Glaeubigerid")
    hrnummer: str | None = Field(default=None, title="Hrnummer")
    kontaktweg: list[Kontaktart] | None = Field(default=None, title="Kontaktweg")
    name1: str = Field(..., title="Name1")
    name2: str | None = Field(default=None, title="Name2")
    name3: str | None = Field(default=None, title="Name3")
    partneradresse: Adresse | None = None
    umsatzsteuer_id: str | None = Field(default=None, alias="umsatzsteuerId", title="Umsatzsteuerid")
    versionstruktur: str | None = Field(default="2", title="Versionstruktur")
    website: str | None = Field(default=None, title="Website")
    erstellungsdatum: datetime | None = Field(default=None, title="Erstellungsdatum")
    geburtstag: datetime | None = Field(default=None, title="Geburtstag")
    telefonnummer_mobil: str | None = Field(default=None, alias="telefonnummerMobil", title="Telefonnummermobil")
    telefonnummer_privat: str | None = Field(default=None, alias="telefonnummerPrivat", title="Telefonnummerprivat")
    telefonnummer_geschaeft: str | None = Field(
        default=None, alias="telefonnummerGeschaeft", title="Telefonnummergeschaeft"
    )
    firmenname: str | None = Field(default=None, title="Firmenname")
    hausbesitzer: bool | None = Field(default=None, title="Hausbesitzer")



from enum import Enum


class Ausschreibungsportal(str, Enum):
    """
    Aufzählung der unterstützten Ausschreibungsportale.
    """

    ENPORTAL = "ENPORTAL"
    ENERGIE_AGENTUR = "ENERGIE_AGENTUR"
    BMWI = "BMWI"
    ENERGIE_HANDELSPLATZ = "ENERGIE_HANDELSPLATZ"
    BUND = "BUND"
    VERA_ONLINE = "VERA_ONLINE"
    ISPEX = "ISPEX"
    ENERGIEMARKTPLATZ = "ENERGIEMARKTPLATZ"
    EVERGABE = "EVERGABE"
    DTAD = "DTAD"



from enum import Enum


class Geraetetyp(str, Enum):
    """
    Auflistung möglicher abzurechnender Gerätetypen.
    """

    WECHSELSTROMZAEHLER = "WECHSELSTROMZAEHLER"
    DREHSTROMZAEHLER = "DREHSTROMZAEHLER"
    ZWEIRICHTUNGSZAEHLER = "ZWEIRICHTUNGSZAEHLER"
    RLM_ZAEHLER = "RLM_ZAEHLER"
    BALGENGASZAEHLER = "BALGENGASZAEHLER"
    MAXIMUMZAEHLER = "MAXIMUMZAEHLER"
    MULTIPLEXANLAGE = "MULTIPLEXANLAGE"
    PAUSCHALANLAGE = "PAUSCHALANLAGE"
    VERSTAERKERANLAGE = "VERSTAERKERANLAGE"
    SUMMATIONSGERAET = "SUMMATIONSGERAET"
    IMPULSGEBER = "IMPULSGEBER"
    EDL_21_ZAEHLERAUFSATZ = "EDL_21_ZAEHLERAUFSATZ"
    VIER_QUADRANTEN_LASTGANGZAEHLER = "VIER_QUADRANTEN_LASTGANGZAEHLER"
    MENGENUMWERTER = "MENGENUMWERTER"
    STROMWANDLER = "STROMWANDLER"
    SPANNUNGSWANDLER = "SPANNUNGSWANDLER"
    KOMBIMESSWANDLER = "KOMBIMESSWANDLER"
    BLOCKSTROMWANDLER = "BLOCKSTROMWANDLER"
    DATENLOGGER = "DATENLOGGER"
    KOMMUNIKATIONSANSCHLUSS = "KOMMUNIKATIONSANSCHLUSS"
    MODEM = "MODEM"
    TELEKOMMUNIKATIONSEINRICHTUNG = "TELEKOMMUNIKATIONSEINRICHTUNG"
    DREHKOLBENGASZAEHLER = "DREHKOLBENGASZAEHLER"
    TURBINENRADGASZAEHLER = "TURBINENRADGASZAEHLER"
    ULTRASCHALLZAEHLER = "ULTRASCHALLZAEHLER"
    WIRBELGASZAEHLER = "WIRBELGASZAEHLER"
    MODERNE_MESSEINRICHTUNG = "MODERNE_MESSEINRICHTUNG"
    INTELLIGENTES_MESSYSTEM = "INTELLIGENTES_MESSYSTEM"
    ELEKTRONISCHER_HAUSHALTSZAEHLER = "ELEKTRONISCHER_HAUSHALTSZAEHLER"
    STEUEREINRICHTUNG = "STEUEREINRICHTUNG"
    TARIFSCHALTGERAET = "TARIFSCHALTGERAET"
    RUNDSTEUEREMPFAENGER = "RUNDSTEUEREMPFAENGER"
    OPTIONALE_ZUS_ZAEHLEINRICHTUNG = "OPTIONALE_ZUS_ZAEHLEINRICHTUNG"
    MESSWANDLERSATZ_IMS_MME = "MESSWANDLERSATZ_IMS_MME"
    KOMBIMESSWANDLER_IMS_MME = "KOMBIMESSWANDLER_IMS_MME"
    TARIFSCHALTGERAET_IMS_MME = "TARIFSCHALTGERAET_IMS_MME"
    RUNDSTEUEREMPFAENGER_IMS_MME = "RUNDSTEUEREMPFAENGER_IMS_MME"
    TEMPERATUR_KOMPENSATION = "TEMPERATUR_KOMPENSATION"
    HOECHSTBELASTUNGS_ANZEIGER = "HOECHSTBELASTUNGS_ANZEIGER"
    SONSTIGES_GERAET = "SONSTIGES_GERAET"
    PREPAYMENTZAEHLER = "PREPAYMENTZAEHLER"
    EDL_21 = "EDL_21"
    FIELD_96_H_ZAEHLER = "96_H_ZAEHLER"
    EDL_40_ZAEHLERAUFSATZ = "EDL_40_ZAEHLERAUFSATZ"
    EDL_40 = "EDL_40"



from pydantic import BaseModel, ConfigDict, Field

from .tarifpreisstaffel_pro_ort import TarifpreisstaffelProOrt


class TarifpreispositionProOrt(BaseModel):
    """
    Mit dieser Komponente können Tarifpreise verschiedener Typen abgebildet werden

    .. raw:: html

        <object data="../_static/images/bo4e/com/TarifpreispositionProOrt.svg" type="image/svg+xml"></object>

    .. HINT::
        `TarifpreispositionProOrt JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/TarifpreispositionProOrt.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    netznr: str | None = Field(default=None, title="Netznr")
    ort: str | None = Field(default=None, title="Ort")
    postleitzahl: str | None = Field(default=None, title="Postleitzahl")
    preisstaffeln: list[TarifpreisstaffelProOrt] | None = Field(default=None, title="Preisstaffeln")



from pydantic import BaseModel, ConfigDict, Field

from .betrag import Betrag
from .kostenposition import Kostenposition


class Kostenblock(BaseModel):
    """
    Mit dieser Komponente werden mehrere Kostenpositionen zusammengefasst.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Kostenblock.svg" type="image/svg+xml"></object>

    .. HINT::
        `Kostenblock JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Kostenblock.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    kostenblockbezeichnung: str | None = Field(default=None, title="Kostenblockbezeichnung")
    kostenpositionen: list[Kostenposition] | None = Field(default=None, title="Kostenpositionen")
    summe_kostenblock: Betrag | None = Field(default=None, alias="summeKostenblock")



from datetime import datetime
from decimal import Decimal

from pydantic import BaseModel, ConfigDict, Field

from ..enum.ablesende_rolle import AblesendeRolle
from ..enum.ablesungsstatus import Ablesungsstatus
from ..enum.mengeneinheit import Mengeneinheit
from ..enum.messwertstatus import Messwertstatus
from ..enum.wertermittlungsverfahren import Wertermittlungsverfahren


class Verbrauch(BaseModel):
    """
    Abbildung eines zeitlich abgegrenzten Verbrauchs

    .. raw:: html

        <object data="../_static/images/bo4e/com/Verbrauch.svg" type="image/svg+xml"></object>

    .. HINT::
        `Verbrauch JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Verbrauch.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    einheit: Mengeneinheit | None = None
    enddatum: datetime | None = Field(default=None, title="Enddatum")
    obis_kennzahl: str = Field(..., alias="obisKennzahl", title="Obiskennzahl")
    startdatum: datetime | None = Field(default=None, title="Startdatum")
    wert: Decimal = Field(..., title="Wert")
    wertermittlungsverfahren: Wertermittlungsverfahren | None = None
    ablesegrund: str | None = Field(default=None, title="Ablesegrund")
    ablesebeschreibung: str | None = Field(default=None, title="Ablesebeschreibung")
    periodenverbrauch: Decimal | None = Field(default=None, title="Periodenverbrauch")
    periodenverbrauch_ursprung: str | None = Field(
        default=None, alias="periodenverbrauchUrsprung", title="Periodenverbrauchursprung"
    )
    ableser: AblesendeRolle | None = None
    status: Ablesungsstatus | None = None
    energiegehalt_gas: Decimal | None = Field(default=None, alias="energiegehaltGas", title="Energiegehaltgas")
    energiegehalt_gas_gueltig_von: datetime | None = Field(
        default=None, alias="energiegehaltGasGueltigVon", title="Energiegehaltgasgueltigvon"
    )
    energiegehalt_gas_gueltig_bis: datetime | None = Field(
        default=None, alias="energiegehaltGasGueltigBis", title="Energiegehaltgasgueltigbis"
    )
    umwandlungsfaktor_gas: Decimal | None = Field(
        default=None, alias="umwandlungsfaktorGas", title="Umwandlungsfaktorgas"
    )
    umwandlungsfaktor_gas_gueltig_von: datetime | None = Field(
        default=None, alias="umwandlungsfaktorGasGueltigVon", title="Umwandlungsfaktorgasgueltigvon"
    )
    umwandlungsfaktor_gas_gueltig_bis: datetime | None = Field(
        default=None, alias="umwandlungsfaktorGasGueltigBis", title="Umwandlungsfaktorgasgueltigbis"
    )
    messwertstatus: Messwertstatus | None = None



from pydantic import BaseModel, ConfigDict, Field

from ..enum.auf_abschlagstyp import AufAbschlagstyp
from ..enum.auf_abschlagsziel import AufAbschlagsziel
from ..enum.waehrungseinheit import Waehrungseinheit
from .energiemix import Energiemix
from .preisgarantie import Preisgarantie
from .regionale_preisstaffel import RegionalePreisstaffel
from .tarifeinschraenkung import Tarifeinschraenkung
from .vertragskonditionen import Vertragskonditionen
from .zeitraum import Zeitraum


class RegionalerAufAbschlag(BaseModel):
    """
    Mit dieser Komponente können Auf- und Abschläge verschiedener Typen im Zusammenhang mit regionalen Gültigkeiten
    abgebildet werden.
    Hier sind auch die Auswirkungen auf verschiedene Tarifparameter modelliert, die sich durch die Auswahl eines Auf-
    oder Abschlags ergeben.

    .. raw:: html

        <object data="../_static/images/bo4e/com/RegionalerAufAbschlag.svg" type="image/svg+xml"></object>

    .. HINT::
        `RegionalerAufAbschlag JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/RegionalerAufAbschlag.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    auf_abschlagstyp: AufAbschlagstyp | None = Field(default=None, alias="aufAbschlagstyp")
    auf_abschlagsziel: AufAbschlagsziel | None = Field(default=None, alias="aufAbschlagsziel")
    beschreibung: str | None = Field(default=None, title="Beschreibung")
    bezeichnung: str | None = Field(default=None, title="Bezeichnung")
    einheit: Waehrungseinheit | None = None
    einschraenkungsaenderung: Tarifeinschraenkung | None = None
    energiemixaenderung: Energiemix | None = None
    garantieaenderung: Preisgarantie | None = None
    gueltigkeitszeitraum: Zeitraum | None = None
    staffeln: list[RegionalePreisstaffel] | None = Field(default=None, title="Staffeln")
    tarifnamensaenderungen: str | None = Field(default=None, title="Tarifnamensaenderungen")
    vertagskonditionsaenderung: Vertragskonditionen | None = None
    voraussetzungen: list[str] | None = Field(default=None, title="Voraussetzungen")
    website: str | None = Field(default=None, title="Website")
    zusatzprodukte: list[str] | None = Field(default=None, title="Zusatzprodukte")



from enum import Enum


class Marktgebiet(str, Enum):
    """
    a strenum where the str-value of each member is the ILN of the Regelzone for Gas
    Also known as: Marktgebietscode
    """

    FIELD_37_Y005053_MH0000_R = "37Y005053MH0000R"
    THE0_BFH027950000 = "THE0BFH027950000"
    THE0_BFL027960000 = "THE0BFL027960000"



from enum import Enum


class Zaehlertyp(str, Enum):
    """
    Bei diesem Enum handelt es sich um die Abbildung von Zählertypen der Sparten Strom und Gas.
    """

    DREHSTROMZAEHLER = "DREHSTROMZAEHLER"
    BALGENGASZAEHLER = "BALGENGASZAEHLER"
    DREHKOLBENZAEHLER = "DREHKOLBENZAEHLER"
    SMARTMETER = "SMARTMETER"
    LEISTUNGSZAEHLER = "LEISTUNGSZAEHLER"
    MAXIMUMZAEHLER = "MAXIMUMZAEHLER"
    TURBINENRADGASZAEHLER = "TURBINENRADGASZAEHLER"
    ULTRASCHALLGASZAEHLER = "ULTRASCHALLGASZAEHLER"
    WECHSELSTROMZAEHLER = "WECHSELSTROMZAEHLER"
    WIRBELGASZAEHLER = "WIRBELGASZAEHLER"
    MESSDATENREGISTRIERGERAET = "MESSDATENREGISTRIERGERAET"
    ELEKTRONISCHERHAUSHALTSZAEHLER = "ELEKTRONISCHERHAUSHALTSZAEHLER"
    SONDERAUSSTATTUNG = "SONDERAUSSTATTUNG"
    MODERNEMESSEINRICHTUNG = "MODERNEMESSEINRICHTUNG"
    LASTGANGZAEHLER = "LASTGANGZAEHLER"
    NEUEMESSEINRICHTUNG = "NEUEMESSEINRICHTUNG"



from pydantic import BaseModel, ConfigDict, Field

from ..bo.marktlokation import Marktlokation
from .angebotsposition import Angebotsposition
from .betrag import Betrag
from .menge import Menge
from .zeitraum import Zeitraum


class Angebotsteil(BaseModel):
    """
    Mit dieser Komponente wird ein Teil einer Angebotsvariante abgebildet.
    Hier werden alle Angebotspositionen aggregiert.
    Angebotsteile werden im einfachsten Fall für eine Marktlokation oder Lieferstellenadresse erzeugt.
    Hier werden die Mengen und Gesamtkosten aller Angebotspositionen zusammengefasst.
    Eine Variante besteht mindestens aus einem Angebotsteil.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Angebotsteil.svg" type="image/svg+xml"></object>

    .. HINT::
        `Angebotsteil JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Angebotsteil.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    anfrage_subreferenz: str | None = Field(default=None, alias="anfrageSubreferenz", title="Anfragesubreferenz")
    gesamtkostenangebotsteil: Betrag | None = None
    gesamtmengeangebotsteil: Menge | None = None
    lieferstellenangebotsteil: list[Marktlokation] | None = Field(default=None, title="Lieferstellenangebotsteil")
    lieferzeitraum: Zeitraum | None = None
    positionen: list[Angebotsposition] | None = Field(default=None, title="Positionen")



from decimal import Decimal

from pydantic import BaseModel, ConfigDict, Field

from .zeitraum import Zeitraum


class Vertragskonditionen(BaseModel):
    """
    Abbildung für Vertragskonditionen. Die Komponente wird sowohl im Vertrag als auch im Tarif verwendet.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Vertragskonditionen.svg" type="image/svg+xml"></object>

    .. HINT::
        `Vertragskonditionen JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Vertragskonditionen.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    abschlagszyklus: Zeitraum | None = None
    anzahl_abschlaege: Decimal | None = Field(default=None, alias="anzahlAbschlaege", title="Anzahlabschlaege")
    beschreibung: str | None = Field(default=None, title="Beschreibung")
    kuendigungsfrist: Zeitraum | None = None
    vertragslaufzeit: Zeitraum | None = None
    vertragsverlaengerung: Zeitraum | None = None



from enum import Enum


class Kundengruppe(str, Enum):
    """
    Kundengruppe für eine Marktlokation (orientiert sich an den Standard-Lastprofilen).
    """

    RLM = "RLM"
    RLM_KOMMUNAL = "RLM_KOMMUNAL"
    SLP_KOMMUNAL = "SLP_KOMMUNAL"
    SLP_S_G0 = "SLP_S_G0"
    SLP_S_G1 = "SLP_S_G1"
    SLP_S_G2 = "SLP_S_G2"
    SLP_S_G3 = "SLP_S_G3"
    SLP_S_G4 = "SLP_S_G4"
    SLP_S_G5 = "SLP_S_G5"
    SLP_S_G6 = "SLP_S_G6"
    SLP_S_G7 = "SLP_S_G7"
    SLP_S_L0 = "SLP_S_L0"
    SLP_S_L1 = "SLP_S_L1"
    SLP_S_L2 = "SLP_S_L2"
    SLP_S_H0 = "SLP_S_H0"
    SLP_S_SB = "SLP_S_SB"
    SLP_S_HZ = "SLP_S_HZ"
    SLP_S_WP = "SLP_S_WP"
    SLP_S_EM = "SLP_S_EM"
    SLP_S_HZ_GEM = "SLP_S_HZ_GEM"
    SLP_G_GKO = "SLP_G_GKO"
    SLP_G_STANDARD = "SLP_G_STANDARD"
    SLP_G_GHA = "SLP_G_GHA"
    SLP_G_GMK = "SLP_G_GMK"
    SLP_G_GBD = "SLP_G_GBD"
    SLP_G_GGA = "SLP_G_GGA"
    SLP_G_GBH = "SLP_G_GBH"
    SLP_G_GBA = "SLP_G_GBA"
    SLP_G_GWA = "SLP_G_GWA"
    SLP_G_GGB = "SLP_G_GGB"
    SLP_G_GPD = "SLP_G_GPD"
    SLP_G_GMF = "SLP_G_GMF"
    SLP_G_HEF = "SLP_G_HEF"
    SLP_G_HMF = "SLP_G_HMF"
    SLP_G_HKO = "SLP_G_HKO"



from enum import Enum


class AufAbschlagstyp(str, Enum):
    """
    Festlegung, ob der Auf- oder Abschlag mit relativen oder absoluten Werten erfolgt.
    """

    RELATIV = "RELATIV"
    ABSOLUT = "ABSOLUT"



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field

from .sepa_info import SepaInfo


class Bankverbindung(BaseModel):
    """
    This component contains bank connection information.
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    iban: str | None = Field(default=None, title="Iban")
    bic: str | None = Field(default=None, title="Bic")
    gueltig_seit: datetime | None = Field(default=None, alias="gueltigSeit", title="Gueltigseit")
    gueltig_bis: datetime | None = Field(default=None, alias="gueltigBis", title="Gueltigbis")
    bankname: str | None = Field(default=None, title="Bankname")
    sepa_info: SepaInfo | None = Field(default=None, alias="sepaInfo")
    kontoinhaber: str | None = Field(default=None, title="Kontoinhaber")
    ouid: int = Field(..., title="Ouid")



from datetime import datetime
from decimal import Decimal

from pydantic import BaseModel, ConfigDict, Field

from ..enum.abgabeart import Abgabeart
from ..enum.energierichtung import Energierichtung
from ..enum.mengeneinheit import Mengeneinheit


class Zaehlwerk(BaseModel):
    """
    Mit dieser Komponente werden Zählwerke modelliert.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Zaehlwerk.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zaehlwerk JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Zaehlwerk.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    bezeichnung: str | None = Field(default=None, title="Bezeichnung")
    einheit: Mengeneinheit | None = None
    obis_kennzahl: str = Field(..., alias="obisKennzahl", title="Obiskennzahl")
    richtung: Energierichtung | None = None
    wandlerfaktor: Decimal | None = Field(default=None, title="Wandlerfaktor")
    zaehlwerk_id: str | None = Field(default=None, alias="zaehlwerkId", title="Zaehlwerkid")
    vorkommastellen: int = Field(..., title="Vorkommastellen")
    nachkommastellen: int = Field(..., title="Nachkommastellen")
    schwachlastfaehig: bool = Field(..., title="Schwachlastfaehig")
    konzessionsabgaben_typ: Abgabeart | None = Field(default=None, alias="konzessionsabgabenTyp")
    active_from: datetime = Field(..., alias="activeFrom", title="Activefrom")
    active_until: datetime | None = Field(default=None, alias="activeUntil", title="Activeuntil")
    description: str | None = Field(default=None, title="Description")
    verbrauchsart: str | None = Field(default=None, title="Verbrauchsart")



from enum import Enum


class Sparte(str, Enum):
    """
    Unterscheidungsmöglichkeiten für die Sparte.
    """

    STROM = "STROM"
    GAS = "GAS"
    FERNWAERME = "FERNWAERME"
    NAHWAERME = "NAHWAERME"
    WASSER = "WASSER"
    ABWASSER = "ABWASSER"
    STROM_UND_GAS = "STROM_UND_GAS"



from enum import Enum


class Vertragsstatus(str, Enum):
    """
    Abbildung einer Statusinformation für Verträge.
    """

    IN_ARBEIT = "IN_ARBEIT"
    UEBERMITTELT = "UEBERMITTELT"
    ANGENOMMEN = "ANGENOMMEN"
    AKTIV = "AKTIV"
    ABGELEHNT = "ABGELEHNT"
    WIDERRUFEN = "WIDERRUFEN"
    STORNIERT = "STORNIERT"
    GEKUENDIGT = "GEKUENDIGT"
    BEENDET = "BEENDET"



from pydantic import BaseModel, ConfigDict, Field

from ..enum.zeiteinheit import Zeiteinheit


class Zeitintervall(BaseModel):
    """
    Abbildung für ein Zeitintervall. Die Abbildung eines Zeitintervalls.
    Z.B. zur Anwendung als Raster in äquidistanten Zeitreihen/Lastgängen, beispielsweise 15 Minuten.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Zeitintervall.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zeitintervall JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Zeitintervall.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    wert: int | None = Field(default=None, title="Wert")
    zeiteinheit: Zeiteinheit | None = None



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field

from ..enum.bdew_artikelnummer import BDEWArtikelnummer
from ..enum.zeiteinheit import Zeiteinheit
from .betrag import Betrag
from .menge import Menge
from .preis import Preis
from .steuerbetrag import Steuerbetrag


class Rechnungsposition(BaseModel):
    """
    Über Rechnungspositionen werden Rechnungen strukturiert.
    In einem Rechnungsteil wird jeweils eine in sich geschlossene Leistung abgerechnet.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Rechnungsposition.svg" type="image/svg+xml"></object>

    .. HINT::
        `Rechnungsposition JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Rechnungsposition.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    artikel_id: str | None = Field(default=None, alias="artikelId", title="Artikelid")
    artikelnummer: BDEWArtikelnummer | None = None
    einzelpreis: Preis | None = None
    lieferung_bis: datetime | None = Field(default=None, alias="lieferungBis", title="Lieferungbis")
    lieferung_von: datetime | None = Field(default=None, alias="lieferungVon", title="Lieferungvon")
    lokations_id: str | None = Field(default=None, alias="lokationsId", title="Lokationsid")
    positions_menge: Menge = Field(..., alias="positionsMenge")
    positionsnummer: int | None = Field(default=None, title="Positionsnummer")
    positionstext: str | None = Field(default=None, title="Positionstext")
    teilrabatt_netto: Betrag | None = Field(default=None, alias="teilrabattNetto")
    teilsumme_netto: Betrag = Field(..., alias="teilsummeNetto")
    teilsumme_steuer: Steuerbetrag = Field(..., alias="teilsummeSteuer")
    zeitbezogene_menge: Menge | None = Field(default=None, alias="zeitbezogeneMenge")
    zeiteinheit: Zeiteinheit | None = None



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field

from .betrag import Betrag
from .menge import Menge
from .preis import Preis


class Kostenposition(BaseModel):
    """
    Diese Komponente wird zur Übertagung der Details zu einer Kostenposition verwendet.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Kostenposition.svg" type="image/svg+xml"></object>

    .. HINT::
        `Kostenposition JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Kostenposition.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    artikelbezeichnung: str | None = Field(default=None, title="Artikelbezeichnung")
    artikeldetail: str | None = Field(default=None, title="Artikeldetail")
    betrag_kostenposition: Betrag | None = Field(default=None, alias="betragKostenposition")
    bis: datetime | None = Field(default=None, title="Bis")
    einzelpreis: Preis | None = None
    menge: Menge | None = None
    positionstitel: str | None = Field(default=None, title="Positionstitel")
    von: datetime | None = Field(default=None, title="Von")
    zeitmenge: Menge | None = None



from pydantic import BaseModel, ConfigDict, Field

from ..enum.kontaktart import Kontaktart
from .adresse import Adresse
from .vertragskonto_cba import VertragskontoCBA


class VertragskontoMBA(BaseModel):
    """
    Models an MBA (master billing account). Its main purpose is to bundle CBAs together having the same address in
    their related contracts. This feature supports a single invoice for all CBAs instead of several
    invoices for each.
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    ouid: int = Field(..., title="Ouid")
    vertrags_adresse: Adresse = Field(..., alias="vertragsAdresse")
    vertragskontonummer: str = Field(..., title="Vertragskontonummer")
    rechnungsstellung: Kontaktart
    cbas: list[VertragskontoCBA] = Field(..., title="Cbas")



from enum import Enum


class Vertragsform(str, Enum):
    """
    Aufzählung der Möglichkeiten zu Vertragsformen in Ausschreibungen.
    """

    ONLINE = "ONLINE"
    DIREKT = "DIREKT"
    FAX = "FAX"



from enum import Enum


class Preismodell(str, Enum):
    """
    Bezeichnung der Preismodelle in Ausschreibungen für die Energielieferung.
    """

    FESTPREIS = "FESTPREIS"
    TRANCHE = "TRANCHE"



from datetime import datetime
from decimal import Decimal

from pydantic import BaseModel, ConfigDict, Field

from ..enum.zeiteinheit import Zeiteinheit


class Zeitraum(BaseModel):
    """
    Diese Komponente wird zur Abbildung von Zeiträumen in Form von Dauern oder der Angabe von Start und Ende verwendet.
    Es muss daher eine der drei Möglichkeiten angegeben sein:
    - Einheit und Dauer oder
    - Zeitraum: Startdatum bis Enddatum oder
    - Zeitraum: Startzeitpunkt (Datum und Uhrzeit) bis Endzeitpunkt (Datum und Uhrzeit)

    .. raw:: html

        <object data="../_static/images/bo4e/com/Zeitraum.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zeitraum JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Zeitraum.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    dauer: Decimal | None = Field(default=None, title="Dauer")
    einheit: Zeiteinheit | None = None
    enddatum: datetime | None = Field(default=None, title="Enddatum")
    endzeitpunkt: datetime | None = Field(default=None, title="Endzeitpunkt")
    startdatum: datetime | None = Field(default=None, title="Startdatum")
    startzeitpunkt: datetime | None = Field(default=None, title="Startzeitpunkt")



from enum import Enum


class Tarifart(str, Enum):
    """
    Die Tarifart wird verwendet zur Charakterisierung von Zählern und daraus resultierenden Tarifen.
    """

    EINTARIF = "EINTARIF"
    ZWEITARIF = "ZWEITARIF"
    MEHRTARIF = "MEHRTARIF"
    SMART_METER = "SMART_METER"
    LEISTUNGSGEMESSEN = "LEISTUNGSGEMESSEN"



from decimal import Decimal

from pydantic import BaseModel, ConfigDict, Field

from ..enum.oekolabel import Oekolabel
from ..enum.oekozertifikat import Oekozertifikat
from ..enum.sparte import Sparte
from .energieherkunft import Energieherkunft


class Energiemix(BaseModel):
    """
    Zusammensetzung der gelieferten Energie aus den verschiedenen Primärenergieformen.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Energiemix.svg" type="image/svg+xml"></object>

    .. HINT::
        `Energiemix JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Energiemix.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    anteil: list[Energieherkunft] | None = Field(default=None, title="Anteil")
    atommuell: Decimal | None = Field(default=None, title="Atommuell")
    bemerkung: str | None = Field(default=None, title="Bemerkung")
    bezeichnung: str | None = Field(default=None, title="Bezeichnung")
    co2_emission: Decimal | None = Field(default=None, alias="co2Emission", title="Co2Emission")
    energieart: Sparte | None = None
    energiemixnummer: int | None = Field(default=None, title="Energiemixnummer")
    gueltigkeitsjahr: int | None = Field(default=None, title="Gueltigkeitsjahr")
    oeko_top_ten: bool | None = Field(default=None, alias="oekoTopTen", title="Oekotopten")
    oekolabel: list[Oekolabel] | None = Field(default=None, title="Oekolabel")
    oekozertifikate: list[Oekozertifikat] | None = Field(default=None, title="Oekozertifikate")
    website: str | None = Field(default=None, title="Website")



from datetime import datetime
from decimal import Decimal

from pydantic import BaseModel, ConfigDict, Field

from ..com.externe_referenz import ExterneReferenz
from ..com.zaehlwerk import Zaehlwerk
from ..com.zeitraum import Zeitraum
from ..enum.bo_typ import BoTyp
from ..enum.messwerterfassung import Messwerterfassung
from ..enum.sparte import Sparte
from ..enum.tarifart import Tarifart
from ..enum.zaehlerauspraegung import Zaehlerauspraegung
from ..enum.zaehlertyp import Zaehlertyp
from .geschaeftspartner import Geschaeftspartner


class Zaehler(BaseModel):
    """
    Object containing information about a meter/"Zaehler".

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Zaehler.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zaehler JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Zaehler.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    bo_typ: BoTyp | None = Field(default=BoTyp.ZAEHLER, alias="boTyp")
    eichung_bis: datetime | None = Field(default=None, alias="eichungBis", title="Eichungbis")
    externe_referenzen: list[ExterneReferenz] | None = Field(
        default=None, alias="externeReferenzen", title="Externereferenzen"
    )
    letzte_eichung: datetime | None = Field(default=None, alias="letzteEichung", title="Letzteeichung")
    sparte: Sparte
    tarifart: Tarifart | None = None
    versionstruktur: str | None = Field(default="2", title="Versionstruktur")
    zaehlerauspraegung: Zaehlerauspraegung | None = None
    zaehlerhersteller: Geschaeftspartner | None = None
    zaehlerkonstante: Decimal | None = Field(default=None, title="Zaehlerkonstante")
    zaehlernummer: str = Field(..., title="Zaehlernummer")
    zaehlertyp: Zaehlertyp | None = None
    zaehlwerke: list[Zaehlwerk] = Field(..., title="Zaehlwerke")
    messwerterfassung: Messwerterfassung | None = Field(default=None, title="Messwerterfassung")
    nachstes_ablesedatum: datetime | None = Field(
        default=None, alias="nachstesAblesedatum", title="Nachstesablesedatum"
    )
    aktiver_zeitraum: Zeitraum | None = Field(default=None, alias="aktiverZeitraum")



from enum import Enum


class Erzeugungsart(str, Enum):
    """
    Auflistung der Erzeugungsarten von Energie.
    """

    FOSSIL = "FOSSIL"
    KWK = "KWK"
    WIND = "WIND"
    SOLAR = "SOLAR"
    KERNKRAFT = "KERNKRAFT"
    WASSER = "WASSER"
    GEOTHERMIE = "GEOTHERMIE"
    BIOMASSE = "BIOMASSE"
    KOHLE = "KOHLE"
    GAS = "GAS"
    SONSTIGE = "SONSTIGE"
    SONSTIGE_EEG = "SONSTIGE_EEG"
    BIOGAS = "BIOGAS"
    KLIMANEUTRALES_GAS = "KLIMANEUTRALES_GAS"



from enum import Enum


class Leistungstyp(str, Enum):
    ARBEITSPREIS_WIRKARBEIT = "ARBEITSPREIS_WIRKARBEIT"
    LEISTUNGSPREIS_WIRKLEISTUNG = "LEISTUNGSPREIS_WIRKLEISTUNG"
    ARBEITSPREIS_BLINDARBEIT_IND = "ARBEITSPREIS_BLINDARBEIT_IND"
    ARBEITSPREIS_BLINDARBEIT_KAP = "ARBEITSPREIS_BLINDARBEIT_KAP"
    GRUNDPREIS = "GRUNDPREIS"
    GRUNDPREIS_ARBEIT = "GRUNDPREIS_ARBEIT"
    GRUNDPREIS_LEISTUNG = "GRUNDPREIS_LEISTUNG"
    MEHRMINDERMENGE = "MEHRMINDERMENGE"
    MESSSTELLENBETRIEB = "MESSSTELLENBETRIEB"
    MESSDIENSTLEISTUNG = "MESSDIENSTLEISTUNG"
    MESSDIENSTLEISTUNG_INKL_MESSUNG = "MESSDIENSTLEISTUNG_INKL_MESSUNG"
    ABRECHNUNG = "ABRECHNUNG"
    KONZESSIONS_ABGABE = "KONZESSIONS_ABGABE"
    KWK_UMLAGE = "KWK_UMLAGE"
    OFFSHORE_UMLAGE = "OFFSHORE_UMLAGE"
    ABLAV_UMLAGE = "ABLAV_UMLAGE"
    SONDERKUNDEN_UMLAGE = "SONDERKUNDEN_UMLAGE"
    REGELENERGIE_UMLAGE = "REGELENERGIE_UMLAGE"
    BILANZIERUNG_UMLAGE = "BILANZIERUNG_UMLAGE"
    AUSLESUNG_ZUSAETZLICH = "AUSLESUNG_ZUSAETZLICH"
    ABLESUNG_ZUSAETZLICH = "ABLESUNG_ZUSAETZLICH"
    ABRECHNUNG_ZUSAETZLICH = "ABRECHNUNG_ZUSAETZLICH"
    SPERRUNG = "SPERRUNG"
    ENTSPERRUNG = "ENTSPERRUNG"
    MAHNKOSTEN = "MAHNKOSTEN"
    INKASSOKOSTEN = "INKASSOKOSTEN"
    EEG_UMLAGE = "EEG_UMLAGE"
    ENERGIESTEUER = "ENERGIESTEUER"
    NETZPREIS = "NETZPREIS"
    MESSPREIS = "MESSPREIS"
    SONSTIGER_PREIS = "SONSTIGER_PREIS"
    DIENSTLEISTUNG = "DIENSTLEISTUNG"



from pydantic import BaseModel, ConfigDict, Field

from ..enum.themengebiet import Themengebiet


class Zustaendigkeit(BaseModel):
    """
    Enthält die zeitliche Zuordnung eines Ansprechpartners zu Abteilungen und Zuständigkeiten.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Zustaendigkeit.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zustaendigkeit JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Zustaendigkeit.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    abteilung: str | None = Field(default=None, title="Abteilung")
    jobtitel: str | None = Field(default=None, title="Jobtitel")
    themengebiet: Themengebiet | None = None



from enum import Enum


class Messart(str, Enum):
    """
    Gibt an, auf welche Art gemessen wurde.
    """

    AKTUELLERWERT = "AKTUELLERWERT"
    MITTELWERT = "MITTELWERT"
    MAXIMALWERT = "MAXIMALWERT"



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field

from ..com.betrag import Betrag
from ..com.externe_referenz import ExterneReferenz
from ..com.rechnungsposition import Rechnungsposition
from ..com.steuerbetrag import Steuerbetrag
from ..com.zeitraum import Zeitraum
from ..enum.bo_typ import BoTyp
from ..enum.rechnungsstatus import Rechnungsstatus
from ..enum.rechnungstyp import Rechnungstyp
from .geschaeftspartner import Geschaeftspartner


class Rechnung(BaseModel):
    """
    Modell für die Abbildung von Rechnungen im Kontext der Energiewirtschaft;
    Ausgehend von diesem Basismodell werden weitere spezifische Formen abgeleitet.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Rechnung.svg" type="image/svg+xml"></object>

    .. HINT::
        `Rechnung JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Rechnung.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    bo_typ: BoTyp | None = Field(default=BoTyp.RECHNUNG, alias="boTyp")
    externe_referenzen: list[ExterneReferenz] | None = Field(
        default=None, alias="externeReferenzen", title="Externereferenzen"
    )
    faelligkeitsdatum: datetime | None = Field(default=None, title="Faelligkeitsdatum")
    gesamtbrutto: Betrag
    gesamtnetto: Betrag | None = None
    gesamtsteuer: Betrag
    original_rechnungsnummer: str | None = Field(
        default=None, alias="originalRechnungsnummer", title="Originalrechnungsnummer"
    )
    rabatt_brutto: Betrag | None = Field(default=None, alias="rabattBrutto")
    rechnungsdatum: datetime | None = Field(default=None, title="Rechnungsdatum")
    rechnungsempfaenger: Geschaeftspartner | None = None
    rechnungsersteller: Geschaeftspartner | None = None
    rechnungsnummer: str | None = Field(default=None, title="Rechnungsnummer")
    rechnungsperiode: Zeitraum | None = None
    rechnungspositionen: list[Rechnungsposition] | None = Field(default=None, title="Rechnungspositionen")
    rechnungsstatus: Rechnungsstatus | None = None
    rechnungstitel: str | None = Field(default=None, title="Rechnungstitel")
    rechnungstyp: Rechnungstyp
    steuerbetraege: list[Steuerbetrag] | None = Field(default=None, title="Steuerbetraege")
    storno: bool = Field(..., title="Storno")
    versionstruktur: str | None = Field(default="2", title="Versionstruktur")
    vorausgezahlt: Betrag | None = None
    zuzahlen: Betrag
    ist_selbstausgestellt: bool | None = Field(default=None, alias="istSelbstausgestellt", title="Istselbstausgestellt")
    ist_reverse_charge: bool | None = Field(default=None, alias="istReverseCharge", title="Istreversecharge")



from enum import Enum


class Angebotsstatus(str, Enum):
    """
    Gibt den Status eines Angebotes an.
    """

    KONZEPTION = "KONZEPTION"
    UNVERBINDLICH = "UNVERBINDLICH"
    VERBINDLICH = "VERBINDLICH"
    BEAUFTRAGT = "BEAUFTRAGT"
    UNGUELTIG = "UNGUELTIG"
    ABGELEHNT = "ABGELEHNT"
    NACHGEFASST = "NACHGEFASST"
    AUSSTEHEND = "AUSSTEHEND"
    ERLEDIGT = "ERLEDIGT"



from enum import Enum


class Geraetemerkmal(str, Enum):
    """
    Auflistung möglicher abzurechnender Gerätetypen.
    """

    EINTARIF = "EINTARIF"
    ZWEITARIF = "ZWEITARIF"
    MEHRTARIF = "MEHRTARIF"
    GAS_G2_5 = "GAS_G2_5"
    GAS_G4 = "GAS_G4"
    GAS_G6 = "GAS_G6"
    GAS_G10 = "GAS_G10"
    GAS_G16 = "GAS_G16"
    GAS_G25 = "GAS_G25"
    GAS_G40 = "GAS_G40"
    GAS_G65 = "GAS_G65"
    GAS_G100 = "GAS_G100"
    GAS_G160 = "GAS_G160"
    GAS_G250 = "GAS_G250"
    GAS_G400 = "GAS_G400"
    GAS_G650 = "GAS_G650"
    GAS_G1000 = "GAS_G1000"
    GAS_G1600 = "GAS_G1600"
    GAS_G2500 = "GAS_G2500"
    IMPULSGEBER_G4_G100 = "IMPULSGEBER_G4_G100"
    IMPULSGEBER_G100 = "IMPULSGEBER_G100"
    TELEFONANSCHLUSS = "TELEFONANSCHLUSS"
    MODEM_GSM = "MODEM_GSM"
    MODEM_GPRS = "MODEM_GPRS"
    MODEM_FUNK = "MODEM_FUNK"
    MODEM_GSM_O_LG = "MODEM_GSM_O_LG"
    MODEM_GSM_M_LG = "MODEM_GSM_M_LG"
    MODEM_FESTNETZ = "MODEM_FESTNETZ"
    MODEM_GPRS_M_LG = "MODEM_GPRS_M_LG"
    PLC_KOM = "PLC_KOM"
    ETHERNET_KOM = "ETHERNET_KOM"
    DSL_KOM = "DSL_KOM"
    LTE_KOM = "LTE_KOM"
    KOMPAKT_MU = "KOMPAKT_MU"
    SYSTEM_MU = "SYSTEM_MU"
    TEMPERATUR_MU = "TEMPERATUR_MU"
    ZUSTANDS_MU = "ZUSTANDS_MU"
    UNBESTIMMT = "UNBESTIMMT"



"""
Contains the dataset for the quantities loader
"""
from ibims.bo4e import Energiemenge, Marktlokation, Marktteilnehmer, Messlokation
from ibims.datasets.base import DataSetBaseModel
from ibims.targetmodels import TransaktionsdatenQuantities


class QuantitiesLoaderDataSet(DataSetBaseModel):
    """
    This is a bo4e dat set that consists out of
    * a Energiemenge
    * a Messlokation or Marktlokation
    * a Marktteilnehmer
    """

    energiemenge: Energiemenge
    """
    The following attributes need to be filled for this DataSet:
    - lokations_id
    - lokationstyp
    - energieverbrauch
    """

    lokation: Messlokation | Marktlokation
    """
    The following attributes need to be filled for this DataSet:
    - messlokations_id/marktlokations_id
    - sparte
    """

    sender: Marktteilnehmer
    """
    The following attributes need to be filled for this DataSet:
    - gewerbekennzeichnung
    - rollencodenummer
    - rollencodetyp
    """

    empfaenger: Marktteilnehmer
    """
    The following attributes need to be filled for this DataSet:
    - gewerbekennzeichnung
    - rollencodenummer
    - rollencodetyp
    """

    transaktionsdaten: TransaktionsdatenQuantities



from pydantic import BaseModel, ConfigDict, Field

from ..com.externe_referenz import ExterneReferenz
from ..com.preisposition import Preisposition
from ..com.zeitraum import Zeitraum
from ..enum.bo_typ import BoTyp
from ..enum.kundengruppe_ka import KundengruppeKA
from ..enum.preisstatus import Preisstatus
from ..enum.sparte import Sparte
from .marktteilnehmer import Marktteilnehmer


class PreisblattKonzessionsabgabe(BaseModel):
    """
    Die Variante des Preisblattmodells zur Abbildung von allgemeinen Abgaben

    .. raw:: html

        <object data="../_static/images/bo4e/bo/PreisblattKonzessionsabgabe.svg" type="image/svg+xml"></object>

    .. HINT::
        `PreisblattKonzessionsabgabe JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/PreisblattKonzessionsabgabe.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    bezeichnung: str | None = Field(default=None, title="Bezeichnung")
    bo_typ: BoTyp | None = Field(default=BoTyp.PREISBLATTKONZESSIONSABGABE, alias="boTyp")
    externe_referenzen: list[ExterneReferenz] | None = Field(
        default=None, alias="externeReferenzen", title="Externereferenzen"
    )
    gueltigkeit: Zeitraum | None = None
    herausgeber: Marktteilnehmer | None = None
    kundengruppe_ka: KundengruppeKA | None = Field(default=None, alias="kundengruppeKA")
    preispositionen: list[Preisposition] | None = Field(default=None, title="Preispositionen")
    preisstatus: Preisstatus | None = None
    sparte: Sparte | None = None
    versionstruktur: str | None = Field(default="2", title="Versionstruktur")



from enum import Enum


class Steuerkennzeichen(str, Enum):
    """
    Zur Kennzeichnung verschiedener Steuersätze und Verfahren.
    """

    UST_0 = "UST_0"
    UST_19 = "UST_19"
    UST_16 = "UST_16"
    UST_7 = "UST_7"
    VST_0 = "VST_0"
    VST_19 = "VST_19"
    VST_7 = "VST_7"
    RCV = "RCV"



from enum import Enum


class Waehrungscode(str, Enum):
    """
    Aufzählung der Währungscodes.
    """

    AED = "AED"
    AFN = "AFN"
    ALL = "ALL"
    AMD = "AMD"
    ANG = "ANG"
    AOA = "AOA"
    ARS = "ARS"
    AUD = "AUD"
    AWG = "AWG"
    AZN = "AZN"
    BAM = "BAM"
    BBD = "BBD"
    BDT = "BDT"
    BGN = "BGN"
    BHD = "BHD"
    BIF = "BIF"
    BMD = "BMD"
    BND = "BND"
    BOB = "BOB"
    BOV = "BOV"
    BRL = "BRL"
    BSD = "BSD"
    BTN = "BTN"
    BWP = "BWP"
    BYN = "BYN"
    BYR = "BYR"
    BZD = "BZD"
    CAD = "CAD"
    CDF = "CDF"
    CHE = "CHE"
    CHF = "CHF"
    CHW = "CHW"
    CLF = "CLF"
    CLP = "CLP"
    CNY = "CNY"
    COP = "COP"
    COU = "COU"
    CRC = "CRC"
    CUC = "CUC"
    CUP = "CUP"
    CVE = "CVE"
    CZK = "CZK"
    DJF = "DJF"
    DKK = "DKK"
    DOP = "DOP"
    DZD = "DZD"
    EGP = "EGP"
    ERN = "ERN"
    ETB = "ETB"
    EUR = "EUR"
    FJD = "FJD"
    FKP = "FKP"
    GBP = "GBP"
    GEL = "GEL"
    GHS = "GHS"
    GIP = "GIP"
    GMD = "GMD"
    GNF = "GNF"
    GTQ = "GTQ"
    GYD = "GYD"
    HKD = "HKD"
    HNL = "HNL"
    HRK = "HRK"
    HTG = "HTG"
    HUF = "HUF"
    IDR = "IDR"
    ILS = "ILS"
    INR = "INR"
    IQD = "IQD"
    IRR = "IRR"
    ISK = "ISK"
    JMD = "JMD"
    JOD = "JOD"
    JPY = "JPY"
    KES = "KES"
    KGS = "KGS"
    KHR = "KHR"
    KMF = "KMF"
    KPW = "KPW"
    KRW = "KRW"
    KWD = "KWD"
    KYD = "KYD"
    KZT = "KZT"
    LAK = "LAK"
    LBP = "LBP"
    LKR = "LKR"
    LRD = "LRD"
    LSL = "LSL"
    LTL = "LTL"
    LYD = "LYD"
    MAD = "MAD"
    MDL = "MDL"
    MGA = "MGA"
    MKD = "MKD"
    MMK = "MMK"
    MNT = "MNT"
    MOP = "MOP"
    MRO = "MRO"
    MUR = "MUR"
    MVR = "MVR"
    MWK = "MWK"
    MXN = "MXN"
    MXV = "MXV"
    MYR = "MYR"
    MZN = "MZN"
    NAD = "NAD"
    NGN = "NGN"
    NIO = "NIO"
    NOK = "NOK"
    NPR = "NPR"
    NZD = "NZD"
    OMR = "OMR"
    PAB = "PAB"
    PEN = "PEN"
    PGK = "PGK"
    PHP = "PHP"
    PKR = "PKR"
    PLN = "PLN"
    PYG = "PYG"
    QAR = "QAR"
    RON = "RON"
    RSD = "RSD"
    RUB = "RUB"
    RUR = "RUR"
    RWF = "RWF"
    SAR = "SAR"
    SBD = "SBD"
    SCR = "SCR"
    SDG = "SDG"
    SEK = "SEK"
    SGD = "SGD"
    SHP = "SHP"
    SLL = "SLL"
    SOS = "SOS"
    SRD = "SRD"
    SSP = "SSP"
    STD = "STD"
    SVC = "SVC"
    SYP = "SYP"
    SZL = "SZL"
    THB = "THB"
    TJS = "TJS"
    TMT = "TMT"
    TND = "TND"
    TOP = "TOP"
    TRY_ = "TRY"
    TTD = "TTD"
    TWD = "TWD"
    TZS = "TZS"
    UAH = "UAH"
    UGX = "UGX"
    USD = "USD"
    USN = "USN"
    USS = "USS"
    UYI = "UYI"
    UYU = "UYU"
    UZS = "UZS"
    VEF = "VEF"
    VND = "VND"
    VUV = "VUV"
    WST = "WST"
    XAF = "XAF"
    XAG = "XAG"
    XAU = "XAU"
    XBA = "XBA"
    XBB = "XBB"
    XBC = "XBC"
    XBD = "XBD"
    XCD = "XCD"
    XDR = "XDR"
    XOF = "XOF"
    XPD = "XPD"
    XPF = "XPF"
    XPT = "XPT"
    XSU = "XSU"
    XTS = "XTS"
    XUA = "XUA"
    XXX = "XXX"
    YER = "YER"
    ZAR = "ZAR"
    ZMW = "ZMW"
    ZWL = "ZWL"



from enum import Enum


class Voraussetzungen(str, Enum):
    """
    Voraussetzungen, die erfüllt sein müssen, damit dieser Tarif zur Anwendung kommen kann.
    """

    EINZUGSERMAECHTIGUNG = "EINZUGSERMAECHTIGUNG"
    ZEITPUNKT = "ZEITPUNKT"
    LIEFERANBINDUNG_EINE = "LIEFERANBINDUNG_EINE"
    LIEFERANBINDUNG_ALLE = "LIEFERANBINDUNG_ALLE"
    GEWERBE = "GEWERBE"
    LASTPROFIL = "LASTPROFIL"
    ZAEHLERTYP_GROESSE = "ZAEHLERTYP_GROESSE"
    AUSSCHLUSS_GROSSVERBRAUCHER = "AUSSCHLUSS_GROSSVERBRAUCHER"
    NEUKUNDE = "NEUKUNDE"
    BESTIMMTE_VERTRAGSFORMALITAETEN = "BESTIMMTE_VERTRAGSFORMALITAETEN"
    SELBSTABLESUNG = "SELBSTABLESUNG"
    ONLINEVORAUSSETZUNG = "ONLINEVORAUSSETZUNG"
    MINDESTUMSATZ = "MINDESTUMSATZ"
    ZUSATZPRODUKT = "ZUSATZPRODUKT"
    NEUKUNDE_MIT_VORAUSSETZUNGEN = "NEUKUNDE_MIT_VORAUSSETZUNGEN"
    DIREKTVERTRIEB = "DIREKTVERTRIEB"
    ANSCHLUSSART = "ANSCHLUSSART"
    ANSCHLUSSWERT = "ANSCHLUSSWERT"
    ALTER_KUNDENANLAGE = "ALTER_KUNDENANLAGE"
    ANLAGEBESCHAFFENHEIT = "ANLAGEBESCHAFFENHEIT"
    BETRIEBSSTUNDENBEGRENZUNG = "BETRIEBSSTUNDENBEGRENZUNG"
    FREIGABEZEITEN = "FREIGABEZEITEN"
    FAMILIENSTRUKTUR = "FAMILIENSTRUKTUR"
    MITGLIEDSCHAFT = "MITGLIEDSCHAFT"
    STAATLICHE_FOERDERUNG = "STAATLICHE_FOERDERUNG"
    BESONDERE_VERBRAUCHSSTELLE = "BESONDERE_VERBRAUCHSSTELLE"
    NIEDRIGENERGIE = "NIEDRIGENERGIE"
    ORTSTEILE_LIEFERGEBIET = "ORTSTEILE_LIEFERGEBIET"
    WAERMEBEDARF_ERDGAS = "WAERMEBEDARF_ERDGAS"
    MAX_ZAEHLER_LIEFERSTELLEN = "MAX_ZAEHLER_LIEFERSTELLEN"
    LIEFERUNGSBESCHRAENKUNG_GASART = "LIEFERUNGSBESCHRAENKUNG_GASART"
    KOMBI_BONI = "KOMBI_BONI"
    ALTVERTRAG = "ALTVERTRAG"
    VORGESCHRIEBENE_ZUSATZANLAGE = "VORGESCHRIEBENE_ZUSATZANLAGE"
    MEHRERE_ZAEHLER_ABNAHMESTELLEN = "MEHRERE_ZAEHLER_ABNAHMESTELLEN"
    BESTIMMTER_ABNAHMEFALL = "BESTIMMTER_ABNAHMEFALL"
    ZUSATZMODALITAET = "ZUSATZMODALITAET"
    NACHWEIS_ZAHLUNGSFAEHIGKEIT = "NACHWEIS_ZAHLUNGSFAEHIGKEIT"
    UMSTELLUNG_ENERGIEART = "UMSTELLUNG_ENERGIEART"



from pydantic import BaseModel, ConfigDict, Field

from ..com.externe_referenz import ExterneReferenz
from ..com.geraeteeigenschaften import Geraeteeigenschaften
from ..com.preisposition import Preisposition
from ..com.zeitraum import Zeitraum
from ..enum.bilanzierungsmethode import Bilanzierungsmethode
from ..enum.bo_typ import BoTyp
from ..enum.dienstleistungstyp import Dienstleistungstyp
from ..enum.netzebene import Netzebene
from ..enum.preisstatus import Preisstatus
from ..enum.sparte import Sparte
from .marktteilnehmer import Marktteilnehmer


class PreisblattMessung(BaseModel):
    """
    Variante des Preisblattmodells zur Abbildung der Preise des Messstellenbetriebs und damit verbundener Leistungen

    .. raw:: html

        <object data="../_static/images/bo4e/bo/PreisblattMessung.svg" type="image/svg+xml"></object>

    .. HINT::
        `PreisblattMessung JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/PreisblattMessung.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    bezeichnung: str | None = Field(default=None, title="Bezeichnung")
    bilanzierungsmethode: Bilanzierungsmethode | None = None
    bo_typ: BoTyp | None = Field(default=BoTyp.PREISBLATTMESSUNG, alias="boTyp")
    externe_referenzen: list[ExterneReferenz] | None = Field(
        default=None, alias="externeReferenzen", title="Externereferenzen"
    )
    gueltigkeit: Zeitraum | None = None
    herausgeber: Marktteilnehmer | None = None
    inklusive_dienstleistungen: list[Dienstleistungstyp] | None = Field(
        default=None, alias="inklusiveDienstleistungen", title="Inklusivedienstleistungen"
    )
    inklusive_geraete: list[Geraeteeigenschaften] | None = Field(
        default=None, alias="inklusiveGeraete", title="Inklusivegeraete"
    )
    messebene: Netzebene | None = None
    preispositionen: list[Preisposition] | None = Field(default=None, title="Preispositionen")
    preisstatus: Preisstatus | None = None
    sparte: Sparte | None = None
    versionstruktur: str | None = Field(default="2", title="Versionstruktur")
    zaehler: Geraeteeigenschaften | None = None



from decimal import Decimal

from pydantic import BaseModel, ConfigDict, Field


class TarifpreisstaffelProOrt(BaseModel):
    """
    Gibt die Staffelgrenzen der jeweiligen Preise an

    .. raw:: html

        <object data="../_static/images/bo4e/com/TarifpreisstaffelProOrt.svg" type="image/svg+xml"></object>

    .. HINT::
        `TarifpreisstaffelProOrt JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/TarifpreisstaffelProOrt.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    arbeitspreis: Decimal | None = Field(default=None, title="Arbeitspreis")
    arbeitspreis_nt: Decimal | None = Field(default=None, alias="arbeitspreisNT", title="Arbeitspreisnt")
    grundpreis: Decimal | None = Field(default=None, title="Grundpreis")
    staffelgrenze_bis: Decimal | None = Field(default=None, alias="staffelgrenzeBis", title="Staffelgrenzebis")
    staffelgrenze_von: Decimal | None = Field(default=None, alias="staffelgrenzeVon", title="Staffelgrenzevon")



from enum import Enum


class Kostenklasse(str, Enum):
    """
    Kostenklassen bilden die oberste Ebene der verschiedenen Kosten.
    In der Regel werden die Gesamtkosten einer Kostenklasse in einer App berechnet.
    """

    FREMDKOSTEN = "FREMDKOSTEN"
    BESCHAFFUNG = "BESCHAFFUNG"
    SELBSTKOSTEN = "SELBSTKOSTEN"
    MARGEN = "MARGEN"
    ENERGIEVERSORGUNGSKOSTEN = "ENERGIEVERSORGUNGSKOSTEN"



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field

from .menge import Menge


class Vertragsteil(BaseModel):
    """
    Abbildung für einen Vertragsteil. Der Vertragsteil wird dazu verwendet,
    eine vertragliche Leistung in Bezug zu einer Lokation (Markt- oder Messlokation) festzulegen.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Vertragsteil.svg" type="image/svg+xml"></object>

    .. HINT::
        `Vertragsteil JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Vertragsteil.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    lokation: str | None = Field(default=None, title="Lokation")
    maximale_abnahmemenge: Menge | None = Field(default=None, alias="maximaleAbnahmemenge")
    minimale_abnahmemenge: Menge | None = Field(default=None, alias="minimaleAbnahmemenge")
    vertraglich_fixierte_menge: Menge | None = Field(default=None, alias="vertraglichFixierteMenge")
    vertragsteilbeginn: datetime | None = Field(default=None, title="Vertragsteilbeginn")
    vertragsteilende: datetime | None = Field(default=None, title="Vertragsteilende")



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field

from ..enum.arithmetische_operation import ArithmetischeOperation


class Messlokationszuordnung(BaseModel):
    """
    Mit dieser Komponente werden Messlokationen zu Marktlokationen zugeordnet.
    Dabei kann eine arithmetische Operation (Addition, Subtraktion, Multiplikation, Division) angegeben werden,
    mit der die Messlokation zum Verbrauch der Marktlokation beiträgt.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Messlokationszuordnung.svg" type="image/svg+xml"></object>

    .. HINT::
        `Messlokationszuordnung JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Messlokationszuordnung.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    arithmetik: ArithmetischeOperation | None = None
    gueltig_bis: datetime | None = Field(default=None, alias="gueltigBis", title="Gueltigbis")
    gueltig_seit: datetime | None = Field(default=None, alias="gueltigSeit", title="Gueltigseit")
    messlokations_id: str = Field(..., alias="messlokationsId", title="Messlokationsid")



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field

from ..com.externe_referenz import ExterneReferenz
from ..com.unterschrift import Unterschrift
from ..com.vertragskonditionen import Vertragskonditionen
from ..enum.bo_typ import BoTyp
from ..enum.sparte import Sparte
from ..enum.vertragsart import Vertragsart
from ..enum.vertragsstatus import Vertragsstatus
from .geschaeftspartner import Geschaeftspartner
from .vertrag import Vertrag


class Buendelvertrag(BaseModel):
    """
    Abbildung eines Bündelvertrags.
    Es handelt sich hierbei um eine Liste von Einzelverträgen, die in einem Vertragsobjekt gebündelt sind.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Buendelvertrag.svg" type="image/svg+xml"></object>

    .. HINT::
        `Buendelvertrag JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Buendelvertrag.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    beschreibung: str | None = Field(default=None, title="Beschreibung")
    bo_typ: BoTyp | None = Field(default=BoTyp.BUENDELVERTRAG, alias="boTyp")
    einzelvertraege: list[Vertrag] | None = Field(default=None, title="Einzelvertraege")
    externe_referenzen: list[ExterneReferenz] | None = Field(
        default=None, alias="externeReferenzen", title="Externereferenzen"
    )
    sparte: Sparte | None = None
    unterzeichnervp1: list[Unterschrift] | None = Field(default=None, title="Unterzeichnervp1")
    unterzeichnervp2: list[Unterschrift] | None = Field(default=None, title="Unterzeichnervp2")
    versionstruktur: str | None = Field(default="2", title="Versionstruktur")
    vertragsart: Vertragsart | None = None
    vertragsbeginn: datetime | None = Field(default=None, title="Vertragsbeginn")
    vertragsende: datetime | None = Field(default=None, title="Vertragsende")
    vertragskonditionen: list[Vertragskonditionen] | None = Field(default=None, title="Vertragskonditionen")
    vertragsnummer: str | None = Field(default=None, title="Vertragsnummer")
    vertragspartner1: Geschaeftspartner | None = None
    vertragspartner2: Geschaeftspartner | None = None
    vertragsstatus: Vertragsstatus | None = None



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field


class SepaInfo(BaseModel):
    """
    This class includes details about the sepa mandates.
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    sepa_id: str = Field(..., alias="sepaId", title="Sepaid")
    sepa_zahler: bool = Field(..., alias="sepaZahler", title="Sepazahler")
    creditor_identifier: str | None = Field(default=None, alias="creditorIdentifier", title="Creditoridentifier")
    gueltig_seit: datetime | None = Field(default=None, alias="gueltigSeit", title="Gueltigseit")



from enum import Enum


class NNRechnungsart(str, Enum):
    """
    Abbildung verschiedener in der INVOIC angegebenen Rechnungsarten.
    """

    HANDELSRECHNUNG = "HANDELSRECHNUNG"
    SELBSTAUSGESTELLT = "SELBSTAUSGESTELLT"



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field

from ..com.externe_referenz import ExterneReferenz
from ..com.lastprofil import Lastprofil
from ..enum.aggregationsverantwortung import Aggregationsverantwortung
from ..enum.bo_typ import BoTyp
from ..enum.profiltyp import Profiltyp
from ..enum.prognosegrundlage import Prognosegrundlage


class Bilanzierung(BaseModel):
    """
    Bilanzierung is a business object used for balancing. This object is no BO4E standard and a complete go
    implementation can be found at
    https://github.com/Hochfrequenz/go-bo4e/blob/3414a1eac741542628df796d6beb43eaa27b0b3e/bo/bilanzierung.go
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    versionstruktur: str | None = Field(default="2", title="Versionstruktur")
    bo_typ: BoTyp | None = Field(default="BILANZIERUNG", alias="boTyp")
    externe_referenzen: list[ExterneReferenz] | None = Field(
        default=None, alias="externeReferenzen", title="Externereferenzen"
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    bilanzierungsbeginn: datetime = Field(..., title="Bilanzierungsbeginn")
    bilanzierungsende: datetime = Field(..., title="Bilanzierungsende")
    bilanzkreis: str | None = Field(default=None, title="Bilanzkreis")
    aggregationsverantwortung: Aggregationsverantwortung | None = None
    lastprofile: list[Lastprofil] | None = Field(default=None, title="Lastprofile")
    prognosegrundlage: Prognosegrundlage | None = None
    details_prognosegrundlage: Profiltyp | None = Field(default=None, alias="detailsPrognosegrundlage")



from pydantic import BaseModel, ConfigDict, Field

from ..com.adresse import Adresse
from ..com.externe_referenz import ExterneReferenz
from ..com.rufnummer import Rufnummer
from ..com.zustaendigkeit import Zustaendigkeit
from ..enum.anrede import Anrede
from ..enum.bo_typ import BoTyp
from ..enum.titel import Titel
from .geschaeftspartner import Geschaeftspartner


class Ansprechpartner(BaseModel):
    """
    Object containing information about a Ansprechpartner

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Ansprechpartner.svg" type="image/svg+xml"></object>

    .. HINT::
        `Ansprechpartner JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Ansprechpartner.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    adresse: Adresse | None = None
    anrede: Anrede | None = None
    bo_typ: BoTyp | None = Field(default=BoTyp.ANSPRECHPARTNER, alias="boTyp")
    e_mail_adresse: str | None = Field(default=None, alias="eMailAdresse", title="Emailadresse")
    externe_referenzen: list[ExterneReferenz] | None = Field(
        default=None, alias="externeReferenzen", title="Externereferenzen"
    )
    geschaeftspartner: Geschaeftspartner | None = None
    individuelle_anrede: str | None = Field(default=None, alias="individuelleAnrede", title="Individuelleanrede")
    kommentar: str | None = Field(default=None, title="Kommentar")
    nachname: str | None = Field(default=None, title="Nachname")
    rufnummer: Rufnummer | None = None
    titel: Titel | None = None
    versionstruktur: str | None = Field(default="2", title="Versionstruktur")
    vorname: str | None = Field(default=None, title="Vorname")
    zustaendigkeit: Zustaendigkeit | None = None



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field

from ..com.externe_referenz import ExterneReferenz
from ..enum.bo_typ import BoTyp
from ..enum.hinweis_thema import HinweisThema


class Hinweis(BaseModel):
    """
    Contains specific hints for the handling of contracts and customers.
    Hints are meant to be read and written by agents or customer service employees.
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    versionstruktur: str | None = Field(default="2", title="Versionstruktur")
    bo_typ: BoTyp | None = Field(default=BoTyp.GESCHAEFTSOBJEKT, alias="boTyp")
    externe_referenzen: list[ExterneReferenz] | None = Field(
        default=None, alias="externeReferenzen", title="Externereferenzen"
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    erstellungsdatum: datetime = Field(..., title="Erstellungsdatum")
    thema: HinweisThema | str = Field(..., title="Thema")
    nachricht: str = Field(..., title="Nachricht")



from pydantic import BaseModel, ConfigDict, Field


class File(BaseModel):
    """
    This class represents a file that is stored in the database.
    """

    model_config = ConfigDict(
        populate_by_name=True,
    )
    file_name_for_docstore: str | None = Field(default=None, title="File Name For Docstore")
    folder_name_for_docstore: str | None = Field(default=None, title="Folder Name For Docstore")
    file: bytes = Field(..., title="File")



from enum import Enum


class Oekolabel(str, Enum):
    """
    Aufzählung der Labels für Öko-Strom von verschiedenen Herausgebern.
    """

    ENERGREEN = "ENERGREEN"
    GASGREEN_GRUENER_STROM = "GASGREEN_GRUENER_STROM"
    GASGREEN = "GASGREEN"
    GRUENER_STROM_GOLD = "GRUENER_STROM_GOLD"
    GRUENER_STROM_SILBER = "GRUENER_STROM_SILBER"
    GRUENER_STROM = "GRUENER_STROM"
    GRUENES_GAS = "GRUENES_GAS"
    NATURWATT_STROM = "NATURWATT_STROM"
    OK_POWER = "OK_POWER"
    RENEWABLE_PLUS = "RENEWABLE_PLUS"
    WATERGREEN = "WATERGREEN"
    WATERGREEN_PLUS = "WATERGREEN_PLUS"



from pydantic import BaseModel, ConfigDict, Field

from ..com.externe_referenz import ExterneReferenz
from ..enum.bo_typ import BoTyp


class Kampagne(BaseModel):
    """
    A "Kampagne"/campaign models which marketing activities led customers to a product/tariff.
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    versionstruktur: str | None = Field(default="2", title="Versionstruktur")
    bo_typ: BoTyp | None = Field(default=BoTyp.GESCHAEFTSOBJEKT, alias="boTyp")
    externe_referenzen: list[ExterneReferenz] | None = Field(
        default=None, alias="externeReferenzen", title="Externereferenzen"
    )
    id: str = Field(..., title="Id")
    name: str = Field(..., title="Name")



from pydantic import BaseModel, ConfigDict, Field

from ..enum.landescode import Landescode


class Adresse(BaseModel):
    """
    Contains an address that can be used for most purposes.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Adresse.svg" type="image/svg+xml"></object>

    .. HINT::
        `Adresse JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Adresse.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    adresszusatz: str | None = Field(default=None, title="Adresszusatz")
    co_ergaenzung: str | None = Field(default=None, alias="coErgaenzung", title="Coergaenzung")
    hausnummer: str | None = Field(default=None, title="Hausnummer")
    landescode: Landescode | None = Landescode.DE
    ort: str = Field(..., title="Ort")
    ortsteil: str | None = Field(default=None, title="Ortsteil")
    postfach: str | None = Field(default=None, title="Postfach")
    postleitzahl: str = Field(..., title="Postleitzahl")
    strasse: str | None = Field(default=None, title="Strasse")



from enum import Enum


class KundengruppeKA(str, Enum):
    """
    Eine Aufzählung zur Einordnung für die Höhe der Konzessionsabgabe.
    """

    S_SCHWACHLAST = "S_SCHWACHLAST"
    S_TARIF_25000 = "S_TARIF_25000"
    S_TARIF_100000 = "S_TARIF_100000"
    S_TARIF_500000 = "S_TARIF_500000"
    S_TARIF_G_500000 = "S_TARIF_G_500000"
    S_SONDERKUNDE = "S_SONDERKUNDE"
    G_KOWA_25000 = "G_KOWA_25000"
    G_KOWA_100000 = "G_KOWA_100000"
    G_KOWA_500000 = "G_KOWA_500000"
    G_KOWA_G_500000 = "G_KOWA_G_500000"
    G_TARIF_25000 = "G_TARIF_25000"
    G_TARIF_100000 = "G_TARIF_100000"
    G_TARIF_500000 = "G_TARIF_500000"
    G_TARIF_G_500000 = "G_TARIF_G_500000"
    G_SONDERKUNDE = "G_SONDERKUNDE"
    SONDER_KAS = "SONDER_KAS"
    SONDER_SAS = "SONDER_SAS"
    SONDER_TAS = "SONDER_TAS"
    SONDER_TKS = "SONDER_TKS"
    SONDER_TSS = "SONDER_TSS"



from enum import Enum


class Rechnungslegung(str, Enum):
    """
    Aufzählung der Möglichkeiten zur Rechnungslegung in Ausschreibungen.
    """

    MONATSRECHN = "MONATSRECHN"
    ABSCHL_MONATSRECHN = "ABSCHL_MONATSRECHN"
    ABSCHL_JAHRESRECHN = "ABSCHL_JAHRESRECHN"
    MONATSRECHN_JAHRESRECHN = "MONATSRECHN_JAHRESRECHN"
    VORKASSE = "VORKASSE"



from pydantic import BaseModel, ConfigDict, Field

from ..com.adresse import Adresse
from ..com.dienstleistung import Dienstleistung
from ..com.externe_referenz import ExterneReferenz
from ..com.geokoordinaten import Geokoordinaten
from ..com.hardware import Hardware
from ..com.katasteradresse import Katasteradresse
from ..enum.bo_typ import BoTyp
from ..enum.netzebene import Netzebene
from ..enum.sparte import Sparte
from .zaehler import Zaehler


class Messlokation(BaseModel):
    """
    Object containing information about a Messlokation

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Messlokation.svg" type="image/svg+xml"></object>

    .. HINT::
        `Messlokation JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Messlokation.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    bo_typ: BoTyp | None = Field(default=BoTyp.MESSLOKATION, alias="boTyp")
    externe_referenzen: list[ExterneReferenz] | None = Field(
        default=None, alias="externeReferenzen", title="Externereferenzen"
    )
    geoadresse: Geokoordinaten | None = None
    geraete: list[Hardware] | None = Field(default=None, title="Geraete")
    grundzustaendiger_msb_codenr: str | None = Field(
        default=None, alias="grundzustaendigerMsbCodenr", title="Grundzustaendigermsbcodenr"
    )
    grundzustaendiger_msbim_codenr: str | None = Field(
        default=None, alias="grundzustaendigerMsbimCodenr", title="Grundzustaendigermsbimcodenr"
    )
    katasterinformation: Katasteradresse | None = None
    messadresse: Adresse | None = None
    messdienstleistung: list[Dienstleistung] | None = Field(default=None, title="Messdienstleistung")
    messgebietnr: str | None = Field(default=None, title="Messgebietnr")
    messlokations_id: str = Field(..., alias="messlokationsId", title="Messlokationsid")
    messlokationszaehler: list[Zaehler] | None = Field(default=None, title="Messlokationszaehler")
    netzebene_messung: Netzebene | None = Field(default=None, alias="netzebeneMessung")
    sparte: Sparte | None = None
    versionstruktur: str | None = Field(default="2", title="Versionstruktur")



from pydantic import BaseModel, ConfigDict, Field

from ..enum.profiltyp import Profiltyp


class Lastprofil(BaseModel):
    """
    This is not part of the official BO4E standard, but is implemented in the c# and go versions:
    https://github.com/Hochfrequenz/BO4E-dotnet/blob/9bdc151170ddba5c9d7535e863d5a396fe7fec52/BO4E/COM/Lastprofil.cs
    https://github.com/Hochfrequenz/go-bo4e/blob/708b39de0dcea8a9448ed4e7341a2687f6bf7c11/com/lastprofil.go
    Fields, which are not needed for migrations, are omitted and the field "profilart" is modelled as Profiltyp ENUM.
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    bezeichnung: str | None = Field(default=None, title="Bezeichnung")
    einspeisung: bool | None = Field(default=False, title="Einspeisung")
    profilart: Profiltyp | None = None



from decimal import Decimal

from pydantic import BaseModel, ConfigDict, Field

from ..enum.mengeneinheit import Mengeneinheit


class Zaehlpunkt(BaseModel):
    """
    The zaehlpunkt object was created during a migration project.
    It contains attributes needed for metering mapping.
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    periodenverbrauch_vorhersage: Decimal = Field(
        ..., alias="periodenverbrauchVorhersage", title="Periodenverbrauchvorhersage"
    )
    einheit_vorhersage: Mengeneinheit | None = Field(default=Mengeneinheit.KWH, alias="einheitVorhersage")
    zeitreihentyp: str | None = Field(default="Z21", title="Zeitreihentyp")
    kunden_wert: Decimal | None = Field(..., alias="kundenWert", title="Kundenwert")
    einheit_kunde: Mengeneinheit | None = Field(default=None, alias="einheitKunde")
    grundzustaendiger: bool | None = Field(default=True, title="Grundzustaendiger")



from enum import Enum


class Aggregationsverantwortung(str, Enum):
    """
    Aggregationsverantwortung describes who is responsible for aggregations. This enum is not part of the official
    bo4e standard, but is already implemented in the c# and go versions:
    https://github.com/Hochfrequenz/BO4E-dotnet/blob/9bdc151170ddba5c9d7535e863d5a396fe7fec52/BO4E/ENUM/Aggregationsverantwortung.cs#L8
    https://github.com/Hochfrequenz/go-bo4e/blob/708b39de0dcea8a9448ed4e7341a2687f6bf7c11/enum/aggregationsverantwortung/aggregationsverantwortung.go
    """

    UENB = "UENB"
    VNB = "VNB"



from pydantic import BaseModel, ConfigDict, Field

from ..com.externe_referenz import ExterneReferenz
from ..com.zeitreihenwert import Zeitreihenwert
from ..enum.bo_typ import BoTyp
from ..enum.medium import Medium
from ..enum.mengeneinheit import Mengeneinheit
from ..enum.messart import Messart
from ..enum.messgroesse import Messgroesse
from ..enum.wertermittlungsverfahren import Wertermittlungsverfahren


class Zeitreihe(BaseModel):
    """
    Abbildung einer allgemeinen Zeitreihe mit einem Wertvektor.
    Die Werte können mit wahlfreier zeitlicher Distanz im Vektor abgelegt sein.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Zeitreihe.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zeitreihe JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Zeitreihe.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    beschreibung: str | None = Field(default=None, title="Beschreibung")
    bezeichnung: str | None = Field(default=None, title="Bezeichnung")
    bo_typ: BoTyp | None = Field(default=BoTyp.ZEITREIHE, alias="boTyp")
    einheit: Mengeneinheit | None = None
    externe_referenzen: list[ExterneReferenz] | None = Field(
        default=None, alias="externeReferenzen", title="Externereferenzen"
    )
    medium: Medium | None = None
    messart: Messart | None = None
    messgroesse: Messgroesse | None = None
    version: str | None = Field(default=None, title="Version")
    versionstruktur: str | None = Field(default="2", title="Versionstruktur")
    werte: list[Zeitreihenwert] | None = Field(default=None, title="Werte")
    wertherkunft: Wertermittlungsverfahren | None = None



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field


class ConcessionFee(BaseModel):
    """
    The Concession Fee object was created during a migration project.
    It contains attributes needed for metering mapping.
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: str | None = Field(default=None, alias="_id", title=" Id")
    market_location_id: str = Field(..., alias="marketLocationId", title="Marketlocationid")
    group: str | None = Field(default=None, title="Group")
    obis: str = Field(..., title="Obis")
    active_from: datetime = Field(..., alias="activeFrom", title="Activefrom")
    active_until: datetime | None = Field(default=None, alias="activeUntil", title="Activeuntil")
    ka: str | None = Field(default=None, title="Ka")



from enum import Enum


class Marktrolle(str, Enum):
    """
    Diese Rollen kann ein Marktteilnehmer einnehmen.
    """

    NB = "NB"
    LF = "LF"
    MSB = "MSB"
    DL = "DL"
    BKV = "BKV"
    BKO = "BKO"
    UENB = "UENB"
    KUNDE_SELBST_NN = "KUNDE_SELBST_NN"
    MGV = "MGV"
    EIV = "EIV"
    RB = "RB"
    KUNDE = "KUNDE"
    INTERESSENT = "INTERESSENT"
    BTR = "BTR"
